@@ -88,13 +88,14 @@ if json:
 if yaml:
     class YAML(ConfigurationProvider):
         extensions = ['yml', 'yaml']
+        dump_args = {'default_flow_style': False}
         def read(self, filename):
             f = file(filename)
             config = yaml.load(f)
             f.close()
             return config
         def _write(self, fp, config):
-            fp.write(yaml.dump(config))
+            fp.write(yaml.dump(config, **self.dump_args))
             # TODO: could use templates to get order down, etc
 
     configuration_providers.append(YAML())
@@ -220,6 +221,7 @@ types = {bool:  BoolCLI(),
          float: FloatCLI(),
          list:  ListCLI(),
          dict:  DictCLI(),
+         str:   BaseCLI(),
          None:  BaseCLI()} # default
 
 __all__ += [i.__class__.__name__ for i in types.values()]
@@ -231,6 +233,15 @@ class Configuration(optparse.OptionParser):
 
     def __init__(self, configuration_providers=configuration_providers, types=types, load=None, dump='--dump', **parser_args):
 
+        # sanity check
+        if isinstance(self.options, dict):
+            self.option_dict = self.options
+        elif isinstance(self.options, list):
+            # XXX could also be tuple, etc
+            self.option_dict = dict(self.options)
+        else:
+            raise NotImplementedError
+
         # setup configuration
         self.config = {}
         self.configuration_providers = configuration_providers
@@ -261,8 +272,11 @@ class Configuration(optparse.OptionParser):
         # add an option for dumping
         formats = self.formats()
         if formats and dump:
-            self.add_option(dump, dest='dump',
-                            help="dump configuration to a file; Formats: %s" % formats)
+            if isinstance(dump, basestring):
+                dump = [dump]
+            dump = list(dump)
+            self.add_option(*dump, dest='dump',
+                            help="Output configuration file; Formats: %s" % formats)
 
 
     ### methods for iteration
@@ -270,7 +284,9 @@ class Configuration(optparse.OptionParser):
 
     def items(self):
         # TODO: allow options to be a list of 2-tuples
-        return self.options.items()
+        if isinstance(self.options, dict):
+            return self.options.items()
+        return self.options
 
     ### methods for validating configuration
 
@@ -280,15 +296,15 @@ class Configuration(optparse.OptionParser):
         """
 
         # ensure options in configuration are in self.options
-        unknown_options = [i for i in config if i not in self.options]
+        unknown_options = [i for i in config if i not in self.option_dict]
         if unknown_options:
             raise UnknownOptionException("Unknown options: %s" % ', '.join(unknown_options))
 
         # ensure options are of the right type (if specified)
         for key, value in config.items():
-            _type = self.options[key].get('type')
-            if _type is None and 'default' in self.options[key]:
-                _type = type(self.options[key]['default'])
+            _type = self.option_dict[key].get('type')
+            if _type is None and 'default' in self.option_dict[key]:
+                _type = type(self.option_dict[key]['default'])
             if _type is not None and not isinstance(value, _type):
                 try:
                     config[key] = _type(value)
@@ -341,13 +357,20 @@ class Configuration(optparse.OptionParser):
     ### XXX could go in a subclass
 
     def cli_formatter(self, option):
-        handler = self.types[self.options[option].get('type')]
+        handler = self.types[self.option_dict[option].get('type')]
         return getattr(handler, 'take_action', lambda x: 1)
 
+    def option_type(self, name):
+        value = self.option_dict[name]
+        if 'type' in value:
+            return value['type']
+        if 'default' in value:
+            return type(value['default'])
+
     def optparse_options(self, parser):
         """add optparse options to a OptionParser instance"""
         for key, value in self.items():
-            handler = self.types[value.get('type')]
+            handler = self.types[self.option_type(key)]
             args, kw = handler(key, value)
             if not args:
                 # No CLI interface
@@ -361,7 +384,7 @@ class Configuration(optparse.OptionParser):
 
         # get CLI configuration options
         cli_config = dict([(key, value) for key, value in options.__dict__.items()
-                           if key in self.options and key in self.parsed])
+                           if key in self.option_dict and key in self.parsed])
 
         # deserialize configuration
         configuration_files = getattr(options, 'load', args)
@@ -382,17 +405,11 @@ class Configuration(optparse.OptionParser):
             # generate configuration
             self(*config)
         except MissingValueException, missingvalues:
+            # errors are handled below
             pass
 
-        # dump configuration, if specified
-        dump = getattr(options, 'dump')
-        if dump:
-            # TODO: have a way of specifying format other than filename
-            self.serialize(dump)
-
-        if missingvalues and not dump:
-            # XXX assuming if you don't have values you were just dumping
-            raise missingvalues
+        # dump configuration
+        dump = self.dump(options, missingvalues)
 
         # update options from config
         options.__dict__.update(self.config)
@@ -400,6 +417,17 @@ class Configuration(optparse.OptionParser):
         # return parsed arguments
         return options, args
 
+    def dump(self, options, missingvalues):
+        """dump configuration, if specified"""
+
+        if missingvalues:
+            self.error(str(missingvalues))
+
+        dump = getattr(options, 'dump')
+        if dump:
+            # TODO: have a way of specifying format other than filename
+            self.serialize(dump)
+
     ### serialization/deserialization
 
     def formats(self):
